@@ -2,9 +2,9 @@
 =============
 
 To simplify the management of function default keyword argument values
-:py:mod:`funconf` introduces two new decorators. The first function
+:py:mod:`funconf` introduces two new decorators. The first decorator 
 :py:func:`wraps_kwargs` makes it trivial to dynamically define the default
-kwargs for a function.  The second function :py:func:`lazy_string_cast`
+kwargs for a function.  The second decorator :py:func:`lazy_string_cast`
 automatically casts *basestring* values to the type of the keyword default
 values found in the function it is wrapping and the type of the values found in
 the *key:value* object passed into its constructor. 
@@ -118,10 +118,9 @@ try:
 except ImportError:
     from funcsigs import signature, Signature, Parameter 
 try:
-    u = unicode
+    basestring = basestring
 except NameError:
-    u = lambda x: x
-from io import StringIO
+    basestring = (str, bytes) 
 import yaml
 
 
@@ -134,14 +133,23 @@ def wraps_kwargs(default_kwargs):
 
         mydict = {a=4, b=2}
         @wraps_kwargs(mydict)
-        def myfunc(**k):
+        def myfunc(a=2, **k):
             pass
 
     The *default_kwargs* object needs to satisfy the *MutableMapping* interface
-    definition. When the wrapped function is called the kwargs passed in are
-    used to update the *default_kwargs* object. The *default_kwargs* object is
-    then passed into the wrapped function i.e. ``wrapped(**default_kwargs)``. 
+    definition. When a wrapped function is called the following transforms
+    occur over *kwargs* before it is passed into the wrapped function:
 
+        1. *default_kwargs* is updated with keywords found in the *kwargs*.
+        2. The functions default keyword arguments that were not in *kwargs*
+           and are defined by *default_kwargs* will be copied into *kwargs*
+           from the *default_kwargs* object.
+        3. If the wrapped function has a variable keyword argument defined 
+           (i.e ``**k``) then the keywords defined by *default_kwargs* that are
+           not defined in *kwargs* will be copied into *kwargs*.
+        4. If the wrapped function had no variable keyword argument defined,
+           then the values found in *kwargs* that are no in the wrapped
+           function's keyword argument list  will be popped from *kwargs*.
 
     :param default_kwargs: kwargs to be fix into the wrapped function.
     :type default_kwargs: mutable mapping
@@ -218,7 +226,7 @@ def lazy_string_cast(model_kwargs={}):
     
     This example demonstrates how :py:func:`lazy_string_cast` can be applied::
         
-        @lazy_string_cast()
+        @lazy_string_cast
         def main(a=4, b=[4, 2, 55]):
             pass
 
@@ -309,11 +317,11 @@ class ConfigSection(MutableMapping):
     
     The following lists the main features of this class:
 
-    * exposes a configuration *option:value* through a standard implementation
-      of the *MutableMapping* abstract type.
-    * when cast to a string it outputs its state in YAML.
-    * as a decorator it utilises the :py:func:`wraps_kwargs` to change
-      the defaults of a variable kwargs function.  
+        1. Exposes a configuration *option:value* through a standard
+           implementation of the *MutableMapping* abstract type.
+        2. When cast to a string it outputs YAML.
+        3. As a decorator it utilises the :py:func:`wraps_kwargs` to change
+           the defaults of a variable kwargs function.  
     """
  
     __slots__ = ('_dirty', '_options', '_section', '_reserved')
@@ -326,8 +334,8 @@ class ConfigSection(MutableMapping):
        
         :param section: defines the name for this section.
         :type section: str
-        :param options: is the mapping containing an initialised set of
-                        *option:values* 
+        :param options: kwargs to initialise this :py:class:`ConfigSection`'s
+                        *option:value* 
         :type options: mutable mapping
         """
         self._section = section
@@ -340,7 +348,7 @@ class ConfigSection(MutableMapping):
 
     def __dir__(self):
         "Return a list of option names and the Base class attributes."
-        return dir(super(ConfigSection, self)) + self._options.keys()
+        return dir(super(ConfigSection, self)) + list(self._options)
 
     def __getattribute__(self, y):
         """Return a option value where y is the *option* name.  Else, return
@@ -402,7 +410,7 @@ class ConfigSection(MutableMapping):
         self._dirty, d = False, self._dirty
         return d
 
-    def __call__(self, func):
+    def __call__(self, func=None, **kwargs):
         """The :py:class:`ConfigSection` object can be used as a function
         decorator.  
 
@@ -421,12 +429,23 @@ class ConfigSection(MutableMapping):
             def func(**k):
                 pass
 
-        :param func: function to be wrapped.
-        :type func: variable kwargs function 
-        :rtype: wrapped function with defined kwargs defaults bound to this 
-                :py:class:`ConfigSection` object.
+        :param func: Decorator parameter. The function or method to be wrapped.
+        :type func: function or method 
+        :param lazy: Factory parameter. Turns lazy_string_cast on or off.
+        :type lazy: Boolean value default True
+        :rtype: As a factory returns decorator function. As a decorator
+                function returns a decorated function. 
         """
-        return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
+        lazy = kwargs.get('lazy', True)
+        def _wraps(func):
+            if lazy:
+                return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
+            else:
+                return wraps_kwargs(self)(func)
+        if inspect.isfunction(func) or inspect.ismethod(func):
+            return _wraps(func)
+        else:
+            return _wraps
 
 
 ConfigSection._reserved = set(dir(ConfigSection))
@@ -438,13 +457,14 @@ class Config(MutableMapping):
     
     The following lists the main features of this class:
 
-    * aggregates :py:class:`ConfigSection` objects that are accessible through
-      attributes.
-    * exposes a translation of the configuration into section_option:value
-      through a standard implementation of the *MutableMapping* abstract type.
-    * when cast to a string it outputs its state in YAML.
-    * as a decorator it utilises the :py:func:`wraps_kwargs` to change
-      the defaults of a variable kwargs function.  
+        1. Aggregates :py:class:`ConfigSection` objects that are accessible
+           through attributes.
+        2. Exposes a translation of the configuration into section_option:value
+           through a standard implementation of the *MutableMapping* abstract
+           type.
+        3. When cast to a string it outputs YAML.
+        4. As a decorator it utilises the :py:func:`wraps_kwargs` to change the
+           defaults of a variable kwargs function.  
     """
 
     __slots__ = ('_sections', '_reserved', '_lookup', '_strict')
@@ -498,8 +518,6 @@ class Config(MutableMapping):
         :param stream: the configuration to be loaded using ``yaml.load``.
         :type stream: stream object
         """
-        if isinstance(stream, basestring):
-            stream = StringIO(u(stream))
         config = yaml.load(stream)
         if not isinstance(config, dict):
             return
@@ -543,7 +561,7 @@ class Config(MutableMapping):
 
     def __dir__(self):
         "Return a list of section names and the Base class attributes."
-        return dir(super(Config, self)) + self._sections.keys()
+        return dir(super(Config, self)) + list(self._sections)
 
     def __getattribute__(self, y):
         """Return a section where y is the *section* name.  Else, return the
@@ -596,7 +614,7 @@ class Config(MutableMapping):
         section = self._sections[s]
         return section[option]
 
-    def __call__(self, func):
+    def __call__(self, func=None, **kwargs):
         """The :py:class:`Config` object can be used as a function decorator.  
         
         Applying this decorator to a function which takes variable kwargs will
@@ -614,13 +632,23 @@ class Config(MutableMapping):
             def func(**k):
                 pass
 
-        :param func: function to be wrapped.
-        :type func: variable kwargs function 
-        :rtype: wrapped function with defined kwargs bound to this
-                :py:class:`Config` object.
+        :param func: Decorator parameter. The function or method to be wrapped.
+        :type func: function or method 
+        :param lazy: Factory parameter. Turns lazy_string_cast on or off.
+        :type lazy: Boolean value default True
+        :rtype: As a factory returns decorator function. As a decorator
+                function returns a decorated function. 
         """
-        return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
-
+        lazy = kwargs.get('lazy', True)
+        def _wraps(func):
+            if lazy:
+                return lazy_string_cast(self)(wraps_kwargs(self)(func)) 
+            else:
+                return wraps_kwargs(self)(func)
+        if inspect.isfunction(func) or inspect.ismethod(func):
+            return _wraps(func)
+        else:
+            return _wraps
 
 Config._reserved = set(dir(Config))
      
